@@ -6,6 +6,7 @@ import pandas as pd
 from typing import Dict, Any
 from jamstats.data.game_data import DerbyGame
 import logging
+import seaborn as sns
 
 logger = logging.Logger(__name__)
 
@@ -22,14 +23,21 @@ def load_json_derby_game(game_json) -> DerbyGame:
     Returns:
         DerbyGame: derby game
     """
+    json_major_version = get_json_major_version(game_json["state"])
     pdf_game_state = json_to_game_dataframe(game_json)
     game_data_dict = extract_game_data_dict(pdf_game_state)
     pdf_roster = extract_roster(pdf_game_state,
                                 game_data_dict["team_1"],
                                 game_data_dict["team_2"])
     pdf_game_data = extract_jam_data(pdf_game_state, pdf_roster)
-    pdf_penalties = extract_penalties(pdf_game_state, pdf_roster)
-    return DerbyGame(pdf_game_data, game_data_dict, pdf_penalties)
+    pdf_penalties = extract_penalties(pdf_game_state, pdf_roster,
+                                      json_major_version)
+    try:
+        pdf_team_colors = extract_team_colors(pdf_game_state)
+    except Exception as e:
+        logger.warn(f"Failed to extract team colors. Exception: {e}")
+
+    return DerbyGame(pdf_game_data, game_data_dict, pdf_penalties, pdf_team_colors)
 
 
 def json_to_game_dataframe(game_json: Dict[Any, Any]) -> pd.DataFrame:
@@ -143,6 +151,11 @@ def extract_game_data_dict(pdf_game_state: pd.DataFrame) -> Dict[str, Any]:
         pdf_game_state.key == team1_key].value)[0]
     team_name_2 = list(pdf_game_state[
         pdf_game_state.key == team2_key].value)[0]
+
+    # Get rid of weird characters in team names
+    team_name_1 = cleanup_team_name(team_name_1)
+    team_name_2 = cleanup_team_name(team_name_2)
+
     return {
         "team_1": team_name_1,
         "team_2": team_name_2
@@ -201,6 +214,8 @@ def extract_jam_data(pdf_game_state: pd.DataFrame,
         else:
             times_to_lead.append(None)
     pdf_jams_summary_withteams["time_to_lead"] = times_to_lead
+
+    logger.debug(f"Made jams summary dataframe. Rows: {len(pdf_jams_summary_withteams)}")
     
     return pdf_jams_summary_withteams
 
@@ -312,9 +327,14 @@ def extract_roster(pdf_game_state: pd.DataFrame,
         team_string_1 = f"Team\(1\)"
         team_string_2 = f"Team\(2\)"
     elif json_major_version == 4:
-        logger.debug("JSON version 5")
+        logger.debug("JSON version 4")
         team_string_1 = f"PreparedTeam\({team_name_1}\)"
         team_string_2 = f"PreparedTeam\({team_name_2}\)"
+    
+    team_string_1 = cleanup_team_name(team_string_1)
+    team_string_2 = cleanup_team_name(team_string_2)
+
+    logger.debug(f"Looking for teams in roster: {team_string_1}, {team_string_2}")
     pdf_game_state_roster = pdf_game_state.loc[
         pdf_game_state.key.str.contains(
             f"ScoreBoard.{team_string_1}.Skater") |
@@ -325,6 +345,8 @@ def extract_roster(pdf_game_state: pd.DataFrame,
     pdf_game_state_roster["team"] = [
         chunks[1][chunks[1].index("(") + 1:chunks[1].index(")")]
         for chunks in pdf_game_state_roster.key_chunks]
+    logger.debug("Roster rows by team:")
+    logger.debug(pdf_game_state_roster.team.value_counts())
     if json_major_version == 5:
         # Version 4 stored the team name. Version 5 stores the number,
         # so translate.
@@ -343,10 +365,10 @@ def extract_roster(pdf_game_state: pd.DataFrame,
     )]
     pdf_roster = pdf_game_state_roster.pivot(index="skater", columns="roster_key", values="value")
 
-    pdf_roster = pdf_game_state_roster.pivot(index="skater", columns="roster_key", values="value")
-    
     skaterid_team_map = dict(zip(*[pdf_game_state_roster["skater"], pdf_game_state_roster["team"]]))
     pdf_roster["team"] = [skaterid_team_map[skater] for skater in pdf_roster.Id]
+    logger.debug("Loaded roster. Skater count by team:")
+    logger.debug(pdf_roster.team.value_counts())
     return pdf_roster
 
 
@@ -391,8 +413,11 @@ def process_team_jam_info(pdf_game_state: pd.DataFrame, team_number: int,
     logger.debug(f"teamjams pivoted rows: {len(pdf_ateamjams_summary)}")
 
     # add jammer info
-    pdf_ateamjams_summary["jammer_id"] = list(pdf_ateamjams_data[
-        pdf_ateamjams_data.key.str.endswith("Fielding(Jammer).Skater")].value)
+    pdf_ateamjams_jammers = pdf_ateamjams_data[
+        pdf_ateamjams_data.key.str.endswith("Fielding(Jammer).Skater")][
+            ["prd_jam", "value"]].rename(columns={"value": "jammer_id"})
+    pdf_ateamjams_summary = pdf_ateamjams_summary.merge(pdf_ateamjams_jammers,
+        on="prd_jam", how="left")
     pdf_ateamjams_summary = pdf_ateamjams_summary.merge(pdf_roster.rename(
         columns={"Id": "jammer_id", "Name": "jammer_name", "RosterNumber": "jammer_number"}),
         on="jammer_id", how="left")
@@ -404,15 +429,17 @@ def process_team_jam_info(pdf_game_state: pd.DataFrame, team_number: int,
     pdf_ateamjams_summary = pdf_ateamjams_summary.merge(
         pdf_jam_skater_lists, on="prd_jam"
     )
+    logger.debug(f"After adding skaters/jam: {len(pdf_ateamjams_summary)}")
 
     pdf_scoringtrips = parse_scoringtrip_data(pdf_ateamjams_data)
     # need to rename the informational columns of pdf_scoringtrips
+    logger.debug(f"Loaded {len(pdf_scoringtrips)} scoring trips.")
     scoringtrip_cols_to_rename = [x for x in pdf_scoringtrips.columns
                                   if x != "prd_jam"]
 
     pdf_ateamjams_summary_withscoringtrips = pdf_ateamjams_summary.merge(
         pdf_scoringtrips, on="prd_jam")
-    logger.debug(f"After adding scoring trips: {len(pdf_ateamjams_summary)}")
+    logger.debug(f"After adding scoring trips: {len(pdf_ateamjams_summary_withscoringtrips)}")
     pdf_ateamjams_summary_kept = pdf_ateamjams_summary_withscoringtrips[
         ["prd_jam"] + TEAMJAM_SUMMARY_COLUMNS + scoringtrip_cols_to_rename]
 
@@ -434,6 +461,7 @@ def extract_team_perjam_skaters(pdf_ateamjams_data: pd.DataFrame,
     Returns:
         pd.DataFrame: dataframe mapping prd_jam to list of skater names for the jam
     """
+    logger.debug("extract_team_perjam_skaters begin")
     pdf_ateamjams_data_fielding = pdf_ateamjams_data[
         pdf_ateamjams_data["keychunk_4"].str.startswith("Fielding")].copy()
     pdf_ateamjams_data_fielding["keychunk_5"] = [
@@ -444,12 +472,15 @@ def extract_team_perjam_skaters(pdf_ateamjams_data: pd.DataFrame,
         "value": "Id"
     })
 
+    logger.debug(f"    Before roster merge, team jam skater data: {len(pdf_ateamjams_data_skaters)}")
     pdf_ateamjams_data_skaters_withname = pdf_ateamjams_data_skaters.merge(
         pdf_roster, on="Id")
+    logger.debug(f"    After roster merge, team jam skater data: {len(pdf_ateamjams_data_skaters_withname)}")
 
     pdf_jam_skater_lists = pdf_ateamjams_data_skaters_withname.groupby(
         "prd_jam")["Name"].apply(list).reset_index()
     pdf_jam_skater_lists = pdf_jam_skater_lists.rename(columns={"Name": "Skaters"})
+    logger.debug(f"    Jam skater lists length: {len(pdf_jam_skater_lists)}")
 
     return pdf_jam_skater_lists
 
@@ -501,7 +532,8 @@ def parse_scoringtrip_data(pdf_ateamjams_data: pd.DataFrame) -> pd.DataFrame:
 
 
 def extract_penalties(pdf_game_state: pd.DataFrame,
-                      pdf_roster: pd.DataFrame) -> pd.DataFrame:
+                      pdf_roster: pd.DataFrame,
+                      json_major_version: int) -> pd.DataFrame:
     """Extract a dataframe with one row per penalty
 
     Args:
@@ -511,18 +543,90 @@ def extract_penalties(pdf_game_state: pd.DataFrame,
     Returns:
         pd.DataFrame: penalty dataframe
     """
-
+    logger.debug("extract_penalties begin")
     pdf_penalty_gamedata = pdf_game_state[(pdf_game_state.n_key_chunks == 5)].copy()
     pdf_penalty_gamedata["keychunk_2"] = [chunks[2] for chunks in pdf_penalty_gamedata.key_chunks]
     pdf_penalty_gamedata["keychunk_3"] = [chunks[3] for chunks in pdf_penalty_gamedata.key_chunks]
     pdf_penalty_gamedata = pdf_penalty_gamedata[
         pdf_penalty_gamedata.keychunk_3.str.startswith("Penalty(")]
+    logger.debug(f"    Rows with `Penalty(`: {len(pdf_penalty_gamedata)}")
     pdf_penalty_gamedata["penalty_key"] = [chunks[4] for chunks in pdf_penalty_gamedata.key_chunks]
     pdf_penalty_gamedata["Id"] = [chunk[len("Skater("):-1]
                                   for chunk in pdf_penalty_gamedata.keychunk_2]
     pdf_penalty_gamedata = pdf_penalty_gamedata.merge(
         pdf_roster[["Id", "Name", "team"]], on="Id")
+    logger.debug(f"    After merging with roster: {len(pdf_penalty_gamedata)}")
     pdf_penalties = pdf_penalty_gamedata[pdf_penalty_gamedata.penalty_key == "Code"]
     pdf_penalties = pdf_penalties.rename(columns={"value": "penalty_code"})
     pdf_penalties = pdf_penalties[["Name", "team", "penalty_code"]]
-    return pdf_penalties
\ No newline at end of file
+    logger.debug(f"    Before merge with penalty codes: {len(pdf_penalties)}")
+
+    # add penalty names
+    pdf_penalty_codes_names = build_penalty_code_name_map(pdf_game_state, json_major_version)
+    pdf_penalties = pdf_penalties.merge(pdf_penalty_codes_names, on="penalty_code")
+    logger.debug(f"Extracted penalties: {len(pdf_penalties)} rows.")
+    return pdf_penalties
+
+
+def build_penalty_code_name_map(pdf_game_state: pd.DataFrame,
+                                json_major_version: int) -> Dict[str, str]:
+    """Pull out the map from penalty codes to names that's repeated in every game file.
+
+    Args:
+        pdf_game_state (pd.DataFrame): game json in dataframe
+
+    Returns:
+        Dict[str, str]: map from letter codes to names
+    """
+    # this code is very inefficient -- will do string matching on the whole game dictionary
+    logger.debug(f"build_penalty_code_name_map begin, version=={json_major_version}")
+    if json_major_version == 5:
+        pdf_penalty_codes = pdf_game_state[
+            pdf_game_state["key"].str.contains("PenaltyCode")].copy()
+    else:
+        pdf_penalty_codes = pdf_game_state[pdf_game_state["key"].str.startswith(
+            "ScoreBoard.PenaltyCodes.Code")].copy()
+    pdf_penalty_codes["penalty_code"] = [x[-2:-1]
+                                        for x in pdf_penalty_codes.key]       
+    logger.debug(f"Built penalty code dataframe: {len(pdf_penalty_codes)} codes")
+    if len(pdf_penalty_codes) == 0:
+        logger.warn("Could not load penalty codes! Penalty plots will fail.")
+    pdf_penalty_codes = pdf_penalty_codes[["penalty_code", "value"]].rename(columns={
+        "value": "penalty_name"
+    })
+    # some of the penalty codes are quite long. Truncate at first comma
+    pdf_penalty_codes["penalty_name"] = [penalty.split(",")[0]
+                                         for penalty in pdf_penalty_codes.penalty_name]
+    return pdf_penalty_codes
+    
+
+def cleanup_team_name(team_name: str) -> str:
+    """ clean up weird characters in team names. I'm not positive this generalizes well.
+    One of our teams had an apostrophe in it, and it was removed in the roster
+    but not elsewhere.
+
+    Args:
+        team_name (str): team name with potentially weird characters
+
+    Returns:
+        stra: cleaned-up team name
+    """
+    return team_name.replace("'", "")
+
+
+def extract_team_colors(pdf_game_data:pd.DataFrame) -> pd.DataFrame:
+    """Extract the team color hex values from the game data
+
+    Args:
+        pdf_game_data (pd.DataFrame): game data
+
+    Returns:
+        pd.DataFrame: dataframe mapping team to color
+    """
+    pdf_team_colors = pdf_game_data[pdf_game_data.key.str.startswith("ScoreBoard.PreparedTeam") &
+                                    pdf_game_data.key.str.endswith("Color(scoreboard_bg)")].copy()
+    pdf_team_colors["team"] = [x[len("ScoreBoard.PreparedTeam("):-len(").Color(scoreboard_bg)")]
+                            for x in pdf_team_colors.key]
+    pdf_team_colors = pdf_team_colors[["team", "value"]].rename(columns={"value": "color"})
+    logger.debug(f"Team colors: {pdf_team_colors}")
+    return pdf_team_colors
\ No newline at end of file
