@@ -13,6 +13,7 @@ from jamstats.plots.jamplots import (
         plot_jammers_by_team,
         plot_lead_summary,
         histogram_jam_duration,
+        plot_team_penalty_counts,
 )
 from jamstats.plots.skaterplots import (
     plot_jammer_stats_team1,
@@ -76,7 +77,13 @@ def make_all_plots(derby_game: DerbyGame,
             f = plot_func(derby_game)
             figures.append(f)
         except Exception as e:
-            logger.warn(f"Failed to make jam plot: {e}")
+            logger.warn(f"Failed to make jam plot {plot_func.__name__}: {e}")
+
+    try:
+        f = plot_team_penalty_counts(derby_game)
+        figures.append(f)
+    except Exception as e:
+            logger.warn(f"Failed to make penalty plots: {e}")
 
     if plot_skaterplots:
         for plot_func in [plot_jammer_stats_team1, plot_jammer_stats_team2,
@@ -85,5 +92,5 @@ def make_all_plots(derby_game: DerbyGame,
                 f = plot_func(derby_game, anonymize_names=anonymize_names)
                 figures.append(f)
             except Exception as e:
-                logger.warn(f"Failed to make skater plot: {e}")
+                logger.warn(f"Failed to make skater plot {plot_func.__name__}: {e}")
     return figures
\ No newline at end of file
