@@ -9,6 +9,9 @@ from matplotlib import pyplot as plt
 import random
 import string
 import pandas as pd
+from matplotlib import gridspec
+from jamstats.plots.plot_util import make_team_color_palette
+
 
 logger = logging.Logger(__name__)
 
@@ -110,59 +113,63 @@ def plot_skater_stats(derby_game: DerbyGame, team_number: int,
     pdf_skater_data = pd.DataFrame({
         "Skater": list(skater_jamcount_map.keys()),
         "Jams": list(skater_jamcount_map.values()),
-    })
+    }).sort_values("Skater")
 
     pdf_team_penalties = derby_game.pdf_penalties[
-        derby_game.pdf_penalties.team == team_name]
-    pdf_team_penalty_counts = pdf_team_penalties.Name.value_counts().reset_index().rename(
-        columns={"index": "Skater", "Name": "Penalties"})
-    # there is for sure a more efficient way to figure out most common penalty.
-    # But I want to make sure I get the lowest-in-alphabet letter
-    skater_mostcommon_penalties = []
-    for skater in pdf_team_penalty_counts.Skater:
-        pdf_skater_penalty_counts = pdf_team_penalties[
-            pdf_team_penalties.Name == skater].penalty_code.value_counts().reset_index().rename(
-                columns={"index": "penalty_code", "penalty_code": "Count"}
-            )
-        penalties_tiedfor_max = list(pdf_skater_penalty_counts[
-            pdf_skater_penalty_counts.Count == max(pdf_skater_penalty_counts.Count)].penalty_code)
-        skater_mostcommon_penalties.append(sorted(penalties_tiedfor_max)[0])
-    pdf_team_penalty_counts["Top Penalty"] = skater_mostcommon_penalties
-    
-
-    pdf_skater_data = pdf_skater_data.merge(
-        pdf_team_penalty_counts, on="Skater", how="left").fillna({
-            "Penalties": 0, "Top Penalty": ""})
+        derby_game.pdf_penalties.team == team_name].copy()
 
     if anonymize_names:
         logger.debug("Anonymizing skater names.")
         name_dict = build_anonymizer_map(set(pdf_skater_data.Skater))
-        pdf_skater_data["Skater"] = [name_dict[skater] for skater in pdf_skater_data.Skater]    
+        pdf_skater_data["Skater"] = [name_dict[skater] for skater in pdf_skater_data.Skater]   
+        pdf_team_penalties["Name"] = [name_dict[skater] for skater in pdf_team_penalties.Name]   
 
-    pdf_skater_data = pdf_skater_data.sort_values("Skater")
+    f, dummy_axis = plt.subplots()
+    dummy_axis.set_xticks([])
+    dummy_axis.set_yticks([])
+    # create grid for different subplots
+    spec = gridspec.GridSpec(ncols=2, nrows=1,
+                             width_ratios=[1, 3], wspace=0)
 
-    f, axes = plt.subplots(1, 2)
-
-    ax = axes[0]
-    sns.barplot(y="Skater", x="Jams", data=pdf_skater_data, ax=ax)
-    ax.set_title("Jams")
+    ax = f.add_subplot(spec[0])
+    sns.barplot(y="Skater", x="Jams", data=pdf_skater_data, ax=ax, color="black")
+    ax.set_title("Jams") 
     ax.set_ylabel("")
 
-    ax = axes[1]
-    sns.barplot(y="Skater", x="Penalties", data=pdf_skater_data, ax=ax)
-    for i, penalty in enumerate(pdf_skater_data["Top Penalty"]):
-        ax.text(.5, i, penalty,
-                horizontalalignment="center",
-                verticalalignment="center")
-    ax.set_title("Penalties (top marked)")
+    try:
+        pdf_team_penalties["Penalty"] = [
+            code + ": " + name
+            for code, name in zip(*[pdf_team_penalties.penalty_code,
+                                    pdf_team_penalties.penalty_name])
+        ]
+        pdf_penalties_long = (
+            pdf_team_penalties.groupby(['Name', 'Penalty']).size().reset_index())
+        a_penalty = list(pdf_team_penalties.Penalty)[0]
+        
+        # add rows for skaters with no penalties
+        skaters_no_penalties = set(pdf_skater_data.Skater).difference(set(set(pdf_team_penalties.Name)))
+        pdf_penalties_long = pd.DataFrame({
+            "Name": list(pdf_penalties_long.Name) + list(skaters_no_penalties),
+            "Penalty": list(pdf_penalties_long.Penalty) + [a_penalty] * len(skaters_no_penalties),
+            0: list(pdf_penalties_long[0]) + [0] * len(skaters_no_penalties)
+        })
+        
+        pdf_penalty_plot =pdf_penalties_long.pivot(columns='Penalty', index='Name', values=0)
+        pdf_penalty_plot = pdf_penalty_plot.sort_values("Name")
 
-    ax.set_yticklabels([])
-    ax.set_ylabel("")
+        ax = f.add_subplot(spec[1])
+        pdf_penalty_plot.plot(kind="barh", stacked=True, ax=ax)
+        plt.gca().invert_yaxis()
+        ax.set_title(f"Penalties by skater")
+        ax.set_ylabel("")
+        ax.set_xlabel("Penalties")
+        ax.set_yticks([])
+    except Exception as e:
+        logger.warn(f"Failed to make skater penalty subplot: {e}")
 
-
-    f.set_size_inches(8, min(2 + len(pdf_skater_data), 11))
+    f.set_size_inches(12, min(2 + len(pdf_skater_data), 11))
     f.suptitle(f"Skater Stats: {team_name}")
-    f.tight_layout() 
+    f.tight_layout()
     return f
 
 
